#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

unsigned char  data[2][160][20];

struct bmp_header {
  unsigned long  filesize;
  unsigned long  reserved1;
  unsigned long  data_offset;
  unsigned long  header_size;
  unsigned long  width;
  unsigned long  height;
  unsigned short  planes;
  unsigned short  bpp;
  unsigned long  comp;
  unsigned long  data_size;
  unsigned long  H_Resolution;
  unsigned long  V_Resolution;
  unsigned long  color_num;
  unsigned long  imp_col_num;
};
typedef struct bmp_header BMPHeader;

struct bmp_palette {
  unsigned char b;
  unsigned char g;
  unsigned char r;
  unsigned char reserved;
};
typedef struct bmp_palette BMPPalette;

int LoadDB( FILE *in, int slot )
{
  char  head[] = "SphereDB";
  int   c;
  short i;
  
  /* First Header */
  for( i=0;i<8;i++ ) {
    c = fgetc( in );
    if( c<0 || (char)c != head[i] ) {
      fprintf( stderr, "It seem Ellegal Database...\nexit.\n" );
      return -1;
    }
  }
  fseek( in, 80-8, SEEK_CUR );
  
  /* Second Header */
  fseek( in, 40, SEEK_CUR );  /* Skip */
  
  /* Set Slot Offset */
  if( slot ) {
    fseek( in, 6400*slot, SEEK_CUR );
  }

  i = fread( data, 1, 6400, in );
  if( i!=6400 ) {
    fprintf( stderr, "Can't Read FullData?!\n" );
    return -1;
  }
  
  return 0;
}

int WriteBMP( FILE *out )
{
  unsigned char  bmpdata[20];
  
  int         i,j;
  BMPHeader   h;
  BMPPalette  pl[2];
  
  /* fseek( out, 0, SEEK_SET ); */ /* Set FilePointer to Start */
  fputc( 'B', out );
  fputc( 'M', out );  /* Header */
  
  h.filesize = 2 + sizeof(BMPHeader) + sizeof(BMPPalette)*2 + 160*20;
  h.reserved1 = 0;
  h.data_offset = 2 + sizeof(BMPHeader) + sizeof(BMPPalette)*2;
  h.header_size = 0x28;
  h.width = 160;
  h.height = 160;
  h.planes = 1;
  h.bpp = 1;
  h.comp = 0;
  h.data_size = 160*20;
  h.H_Resolution = 0;
  h.V_Resolution = 0;
  h.color_num = 0;
  h.imp_col_num = 0;
  fwrite( &h, sizeof(BMPHeader), 1, out );
  pl[0].r = 255; pl[0].g = 255; pl[0].b = 255; pl[0].reserved = 0;
  pl[1].r = 0;   pl[1].g = 0;   pl[1].b = 0;   pl[1].reserved = 0;
  fwrite( pl, sizeof(BMPPalette), 2, out );
  
  for( i=0;i<160;i++ ) {
    for( j=0; j<20; j++ ) {
      bmpdata[j] = data[0][159-i][j] | data[1][159-i][j];
    }
    fwrite( bmpdata, 1, 20, out );
  }
  
  return 0;
}

int main( int n, char *args[] )
{
  int  result,i;
  
  int  slot,c;
  char *pdbname,*outname;
  FILE *pdbfile,*outfile;
  
  slot = 0;
  pdbname = NULL;
  
  if( n<2 ) {
    printf( "usage: sphereconv [-d <pdbfile>] [-s <slot>] <output.bmp>\n" );
    exit( 0 );
  }
  
  /* Option */
  while( (c = getopt( n, args, "d:s:" )) != -1 ) {
    switch( (char)c ) {
    case 'd': /* pdbfile */
      pdbname = optarg;
      break;
    case 's': /* Slot */
      slot = atoi( optarg );
      break;
    }
  }
  outname = args[optind];
  
  if( pdbname == NULL ) {
    pdbname = "SphereDB.pdb";
  }
  
  if( outname == NULL ) {
    fprintf( stderr, "Prease Tell me Output Filename...\n" );
  }
  
  if( pdbname == NULL ) {
    pdbname = "SphereDB.pdb";
  }
  
  pdbfile = fopen( pdbname, "rb" );
  if( pdbfile == NULL ) {
    fprintf( stderr, "Can't Open PDBFile \"SphereDB.pdb\" !!\n" );
    exit( 1 );
  }
  
  result = LoadDB( pdbfile, slot );
  
  outfile = fopen( outname, "wb" );
  if( outfile == NULL ) {
    fprintf( stderr, "Can't Output BMPFile...\n" );
    exit( 1 );
  }
  WriteBMP( outfile ); 
  
  fprintf( stdout, "Convert Success!!\n" );
  
  return 0;
}
